*Deck DoMP2
      Program DoMP2
      Implicit Real*8(A-H,O-Z)
C
C     A sample program which reads eigenvalues and MO two-electron
C     integrals from a matrix element file and computes the MP2
C     energy.
C
      Logical ASym, EOF, Debug
C     Fixed maximum dimensions to keep things simple.
      Parameter (MaxAt=20,MaxBas=100,MaxB2=(MaxBas*(MaxBas+1))/2,
     $  MaxGSc=1000,Debug=.False.)
      Real*8 AtmChg(MaxAt), C(3,MaxAt), AtmWgt(MaxAt), AOrbEn(MaxBas),
     $  AAMO2E(MaxBas,MaxBas,MaxBas,MaxBas), TmpMO(MaxB2,MaxB2),
     $  BAMO2E(MaxBas,MaxBas,MaxBas,MaxBas), GScal(MaxGSc),
     $  BBMO2E(MaxBas,MaxBas,MaxBas,MaxBas), BOrbEn(MaxBas)
      Integer IAn(MaxAt), IAtTyp(MaxAt), IBfAtm(MaxBas), IBfTyp(MaxBas)
      Character*80 FileName, LabFil, GVers, Title*1024, CBuf*64
 1000 Format(' Label "',A,'"',/,' program version "',A,'"',/,' title "',
     $  A,'"',/,' NAtoms=',I6,' NBasis=',I6,' NBsUse=',I6,' ICharg=',I3,
     $  ' Multip=',I3,' NE=',I6,' Len12L=',I1,' Len4L=',I1,' IOpCl=',i4,
     $  ' ICGU=',I4,'.')
 1010 Format(' NAE=',I5,' NBE=',I5,' NOA=',I5,' NOB=',I5,' NVA=',I5,
     $  ' NVB=',I5,' ITran=',I1,'.')
 1020 Format(' Skip matrix labelled "',A,'".')
 1030 Format(' Alpha orbital energies=',5F14.6)
 1040 Format(' Beta orbital energies=',5F14.6)
 1050 Format(1X,A,' T2=',F14.6,' E2=',F14.6)
 1060 Format(' EHF=',F14.6,' E2=',F14.6,' EMP2=',F14.6)
C
      FileName = ' '
      Call Get_Command_Argument(1,FileName,LFN,IStat)
      If(IStat.ne.0) then
        Write(6,*) 'Error fetching argument, status=',IStat
        Stop
        endIf
      Write(6,*) 'file "',FileName(1:LFN),'"'
C
C     Open the matrix element file.  IUnit is returned with the Fortran
C     unit number used to open the file, along with parameters giving
C     the basic structure and dimensions fo rthe file.
C
      LabFil = ' '
      GVers = ' '
      Title = ' '
      Call Open_Read(FileName(1:LFN),IUnit,LabFil,IVers,NLab,GVers,
     $  Title,NAtoms,NBasis,NBsUse,ICharg,Multip,NE,Len12L,Len4L,IOpCl,
     $  ICGU)
      If(IU.lt.1) then
        Write(6,*) 'failed to open file'
        Stop
        endIf
C
C     Head the header records, with basic data about the molecule,
c     including atomic numbers and coordinates and information about
C     the basis set (none of which are needed here).
C
      LLF = Len_Trim(LabFil)
      LGV = Len_Trim(GVers)
      LTit = Len_Trim(Title)
      Write(6,1000) LabFil(1:LLF), GVers(1:LGV), Title(1:LTit), NAtoms,
     $  NBasis, NBsUse, ICharg, Multip, NE, Len12L, Len4L, IOpCl, ICGU
      Call Rd_Head(IUnit,NLab,NAtoms,NBasis,IAn,IAtTyp,AtmChg,C,
     $  IBfAtm,IBfTyp,AtmWgt,NFC,NFV,ITran,IDum9,NShlAO,NPrmAO,NShlDB,
     $  NPrmDB,NBTot)
C
C     Determine the total numbers of alpha and beta electrons and
C     the active (non-frozen-core) numbers, along with the total
C
      NAE = (NE+Multip-1)/2
      NBE = (NE-Multip+1)/2
      NOA = NAE - NFC
      NOB = NBE - NFC
C     NBsUse = number of linearly independent basis functions.
C     NROrb = number of active (non-frozen-core) orbitals
      NROrb = NBsUse - NFC - NFV
      NVA = NROrb - NOA
      NVB = NROrb - NOB
      Write(6,1010) NAE, NBE, NOA, NOB, NVA, NVB, ITran
      NOTT = (NROrb*(NROrb+1))/2
      AOrbEn = 0.0d0
      BOrbEn = 0.0d0
      GScal = -100.0d0
C
C     Read through the data elements in the file, picking up the
C     orbital energies, the MO 2e integrals, and the scalars with
C     various energy quantities and skipping the rest.  ITran=5
C     flags a full transformation, while ITran=4 flags a partial
C     transformation (one index occupieds, the others occupied or
C     virtual).  The 2e integrals are expanded to full 4-index
C     arrays.
C
   10 CBuf = ' '
      Call Rd_Labl(IUnit,IVers,CBuf,NI,NR,NTot,LenBuf,N1,N2,N3,N4,N5,
     $  ASym,NRI,EOF)
      If(.not.EOF) then
        LCB = Len_Trim(CBuf)
        If(CBuf(1:LCB).eq.'ALPHA ORBITAL ENERGIES') then
          Call Rd_RBuf(IUnit,NR*NTot,NR*LenBuf,AOrbEn)
        else if(CBuf(1:LCB).eq.'BETA ORBITAL ENERGIES') then
          Call Rd_RBuf(IUnit,NR*NTot,NR*LenBuf,BOrbEn)
        else if(CBuf(1:LCB).eq.'GAUSSIAN SCALARS') then
C         The scalars are stored packed, with integers giving
C         which non0-zero values are present.
          Call Rd_RInd(IUnit,NR,LR,NTot,LenBuf,LNZ,GScal)
        else if(CBuf(1:LCB).eq.'AA MO 2E INTEGRALS') then
          Call Rd_RBuf(IUnit,NR*NTot,NR*LenBuf,TmpMO)
          If(ITran.eq.5) then
C           A full transformation, NROrb^4/8 integrals
            Call Expnd4(MaxBas,NROrb,TmpMO,AAMO2E)
          else
C           A partial transformation.
            Call Expnd1(.True.,MaxBas,NROrb,NOTT,NOA,TmpMO,AAMO2E)
            endIf
        else if(CBuf(1:LCB).eq.'BA MO 2E INTEGRALS') then
          Call Rd_RBuf(IUnit,NR*NTot,NR*LenBuf,TmpMO)
          If(ITran.eq.5) then
C           (bb|aa) integrals packed in lower triangles.
            Call Expnd2(MaxBas,NROrb,NOTT,TmpMO,BAMO2E)
          else
C           (bb|aa) with only occupieds for slowest a index.
            Call Expnd1(.False.,MaxBas,NROrb,NOTT,NOA,TmpMO,BAMO2E)
            endIf
        else if(CBuf(1:LCB).eq.'BB MO 2E INTEGRALS') then
          Call Rd_RBuf(IUnit,NR*NTot,NR*LenBuf,TmpMO)
          If(ITran.eq.5) then
C           A full transformation, NROrb^4/8 integrals
            Call Expnd4(MaxBas,NROrb,TmpMO,BBMO2E)
          else
C           A partial transformation.
            Call Expnd1(.True.,MaxBas,NROrb,NOTT,NOB,TmpMO,BBMO2E)
            endIf
        else
          If(Debug) Write(6,1020) CBuf(1:LCB)
          Call Rd_Skip(IUnit,NTot,LenBuf)
          endIf
        Goto 10
        endIf
      If(Debug) then
        Write(6,1030) (AOrbEn(I),I=1,NBsUse)
        If(IOpCl.eq.1) Write(6,1040) (BOrbEn(I),I=1,NBsUse)
        endIf
      Call Close_MatF(IUnit)
C
C     Compute E2 contributions from the spin-cases.
C
      Call DoMP2_XX(MaxBas,NFC,NOA,NVA,AOrbEn,AAMO2E,E2AA,T2AA)
      If(IOpCl.eq.0) then
        Call DoMP2_XY(MaxBas,NFC,NOA,NOA,NVA,NVA,AOrbEn,AOrbEn,AAMO2E,
     $    E2AB,T2AB)
        E2BB = E2AA
        T2BB = T2AA
      else
        Call DoMP2_XY(MaxBas,NFC,NOA,NOB,NVA,NVB,AOrbEn,BOrbEn,BAMO2E,
     $    E2AB,T2AB)
        Call DoMP2_XX(MaxBas,NFC,NOB,NVB,BOrbEn,BBMO2E,E2BB,T2BB)
        endIf
      Write(6,1050) 'AA', T2AA, E2AA
      Write(6,1050) 'AB', T2AB, E2AB
      Write(6,1050) 'BB', T2BB, E2BB
      E2 = E2AA + E2AB + E2BB
      EHF = GScal(32)
      ETot = E2 + EHF
      Write(6,1060) EHF, E2, ETot
      End
*Deck Expnd1
      Subroutine Expnd1(DoBK,MaxBas,NROrb,NOTT,NO,TmpMO,XXMO2E)
      Implicit Real*8(A-H,O-Z)
C
C     Expand a set of MO integrals from a partial transformation
C     from (NOTT,NROrb,NO) to full storage.  DoBK is true if the
C     bra and ket should be symmetried (i.e., the same-spin case).
C
      Logical DoBK
      Dimension TmpMO(NOTT,NROrb,*), XXMO2E(MaxBas,MaxBas,MaxBas,*)
C
      Do 40 I = 1, NO
        Do 30 J = 1, NROrb
          Do 20 K = 1, NROrb
            KK = (K*(K-1))/2
            Do 10 L = 1, K
              X = TmpMO(KK+L,J,I)
              XXMO2E(L,K,J,I) = X
              XXMO2E(K,L,J,I) = X
              XXMO2E(L,K,I,J) = X
   10         XXMO2E(K,L,I,J) = X
   20       Continue
   30     Continue
   40   Continue
      If(DoBK) then
        Do 80 I = 1, NO
          Do 70 J = 1, NROrb
            Do 60 K = 1, NROrb
              Do 50 L = 1, NROrb
                X = XXMO2E(L,K,J,I)
                XXMO2E(J,I,L,K) = X
                XXMO2E(J,I,K,L) = X
                XXMO2E(I,J,L,K) = X
   50           XXMO2E(I,J,K,L) = X
   60         Continue
   70       Continue
   80     Continue
        endIf
      Return
      End
*Deck Expnd2
      Subroutine Expnd2(MaxBas,NROrb,NOTT,TmpMO,BAMO2E)
      Implicit Real*8(A-H,O-Z)
C
C     Expand the mixed-spin MO 2e integrals from a full
C     transformation -- spins (bb|aa) -- from lower
C     triangles on the bra and ket to full storage.
C
      Dimension TmpMO(NOTT,*), BAMO2E(MaxBas,MaxBas,MaxBas,*)
C
      Do 40 I = 1, NROrb
        II = (I*(I-1))/2
        Do 30 J = 1, I
          IJ = II + J
          Do 20 K = 1, NROrb
            KK = (K*(K-1))/2
            Do 10 L = 1, K
              X = TmpMO(KK+L,IJ)
              BAMO2E(L,K,J,I) = X
              BAMO2E(K,L,J,I) = X
              BAMO2E(L,K,I,J) = X
   10         BAMO2E(K,L,I,J) = X
   20       Continue
   30     Continue
   40   Continue
      Return
      End
*Deck Expnd4
      Subroutine Expnd4(MaxBas,NROrb,TmpMO,XXMO2E)
      Implicit Real*8(A-H,O-Z)
C
C     Expand the pure-spin MO 2e integrals from a full
C     transformation from canonical order to full N^4.
C
      Dimension TmpMO(*), XXMO2E(MaxBas,MaxBas,MaxBas,*)
C
      Ind = 0
      Do 40 I = 1, NROrb
        Do 30 J = 1, I
          Do 20 K = 1, I
            If(K.eq.I) then
              LimL = J
            else
              LimL = K
              endIf
            Do 10 L = 1, LimL
              Ind = Ind + 1
              X = TmpMO(Ind)
              XXMO2E(L,K,J,I) = X
              XXMO2E(K,L,J,I) = X
              XXMO2E(L,K,I,J) = X
              XXMO2E(K,L,I,J) = X
              XXMO2E(J,I,L,K) = X
              XXMO2E(J,I,K,L) = X
              XXMO2E(I,J,L,K) = X
   10         XXMO2E(I,J,K,L) = X
   20       Continue
   30     Continue
   40   Continue
      Return
      End
*Deck DoMP2_XX
      Subroutine DoMP2_XX(MaxBas,NFC,NO,NV,XOrbEn,XXMO2E,E2,T2)
      Implicit Real*8(A-H,O-Z)
C
C     Compute a same-spin contributions to E2.
C
      Dimension XOrbEn(*), XXMO2E(MaxBas,MaxBas,MaxBas,MaxBas)
      Integer A, B, AA, BB
C
      E2 = 0.0d0
      T2 = 0.0d0
      Do 40 I = 1, NO
        Do 30 J = 1, (I-1)
          EIJ = XOrbEn(I+NFC) + XOrbEn(J+NFC)
          Do 20 A = 1, NV
            AA = A + NO
            EIJA = XOrbEn(AA+NFC) - EIJ
            Do 10 B = 1, (A-1)
              BB = B + NO
              EIJAB = XOrbEn(BB+NFC) + EIJA
              TEI = XXMO2E(BB,J,AA,I) - XXMO2E(BB,I,AA,J)
              E2 = E2 - TEI*TEI/EIJAB
   10         T2 = T2 + TEI*TEI/EIJAB**2
   20       Continue
   30     Continue
   40   Continue
      Return
      End
*Deck DoMP2_XY
      Subroutine DoMP2_XY(MaxBas,NFC,NOA,NOB,NVA,NVB,AOrbEn,BOrbEn,
     $  BAMO2E,E2,T2)
      Implicit Real*8(A-H,O-Z)
C
C     Compute the mixed-spin contribution to E2.
C
      Real*8 AOrbEn(*), BOrbEn(*), BAMO2E(MaxBas,MaxBas,MaxBas,MaxBas)
      Integer A, B, AA, BB
C
      E2 = 0.0d0
      T2 = 0.0d0
      Do 40 I = 1, NOA
        Do 30 J = 1, NOB
          EIJ = AOrbEn(I+NFC) + BOrbEn(J+NFC)
          Do 20 A = 1, NVA
            AA = A + NOA
            EIJA = AOrbEn(AA+NFC) - EIJ
            Do 10 B = 1, NVB
              BB = B + NOB
              EIJAB = BOrbEn(BB+NFC) + EIJA
              TEI = BAMO2E(BB,J,AA,I)
              E2 = E2 - TEI*TEI/EIJAB
   10         T2 = T2 + TEI*TEI/EIJAB**2
   20       Continue
   30     Continue
   40   Continue
      Return
      End
