#!/usr/bin/env perl
#
# This script tests the perl interface to raw matrix element files.
#
# $1 -- name of input raw file
# $2 -- 0/1 don't/do print values in arrays
# $3 -- name of output raw file
#
use strict;
use MatEl '%scalars';
use OpMat;
use GauComplex;

my ($me) = MatEl->new(2);
my ($x,$val,$y,$type,$i);

print "file $ARGV[0]\n";
$me->read($ARGV[0]);

my ($doval,$vals,$lvals,$lvalsx) = $ARGV[1];

my ($x,$y,$y1,$dims,$lenv);

my ($js) = $me->item("job status");
print "Job Status $js\n";

foreach $x ("IAN", "IATTYP", "ATMCHG", "C", "IBF", "IBFTYP") {
   $y = $me->$x;
   $y1 = $me->item($x);
   print "$x ", join(" ",@$y), "\n";
   print "$x ", join(" ",@$y1), "\n";
}

$y = $me->item("gaussian scalars");
for $x (keys(%scalars)) {
  $i = $scalars{$x};
  if ($y->[$i] != 0.0e0) {print "gen $i $x $y->[$i]\n";}
}

print "NATOMS " . $me->NATOMS . "\n";
print "ETOTAL " . $me->ETOTAL . "\n";
$me->ETOTAL(1.234);
print "ETOTAL " . $me->ETOTAL . "\n";
print "2ESCFENERGY " . $me->TESCFENERGY . "\n";

#GauComplex->display_format('style' => 'polar');

foreach $x (sort keys %$me) {
  print "key $x ";
  $val = $$me{$x};
  if (ref($val) =~ /^OpMat/) {
    $lvals = $val->lenarray(1);
    $type = $val->type();
    $dims = $val->dimens();
    $vals = $val->array(1);
    $lvalsx = @$vals;
    print "type $type len $lvals $lvalsx dims @$dims";
    if ($doval) {
      print " xvalues ";
      foreach $y (@$vals) {print "$y"," ";}
    }
    print "\n";}
  elsif (ref($val) eq "ARRAY") {
    print "array len ". @$val . "\n"; 
    if ($doval) {print "  values ",join(" ",@$val),"\n";}}
  else {print "key $x val $val\n";}
}

#$me->_makecon();
$me->prtcon();

my ($amo) = $me->item("alpha mo coefficients");
my ($ovl) = $me->item("overlap");

my ($amodim) = $amo->dimens;

if ($ARGV[2]) {
  print "got ",ref($amo),"\n";
  my ($ind,$val,$nval);
  $ind = $amo->make_indx(2,3); print "ind 2,3 is $ind\n";
  $ind = $amo->make_indx(2,4); print "ind 2,4 is $ind\n";
  $ind = $amo->make_indx(3,4); print "ind 3,4 is $ind\n";

  print "mo23 is ",$amo->get_elem(2,3),"\n";
  print "mo32 is ",$amo->get_elem(3,2),"\n";

  $val = $amo->get_elem(3,2);

  if (ref($val) eq "GauComplex") {$nval = cplx(2.5,3.5);}
  else {$nval = 2.5;}

  $amo->set_elem($nval,3,2);

  print "now mo32 is ",$amo->get_elem(3,2),"\n";

  if (defined($ovl)) {
    print "ov34 is ",$ovl->get_elem(3,4),"\n";
    $ovl->set_elem($nval,3,4);
    print "now ov34 is ",$ovl->get_elem(3,4),"\n";}

  $me->title("New title");

  $me->write($ARGV[2]);

  $amo = $me->overlap;

  if (defined($amo)) {print "ov23 is ",$amo->make_indx(2,3),"\n";}
}
else {
  $x = $me->IBF;
  print "IBF @$x\n";
  my (@y) = (1,27);
  $me->ibf(\@y);
  $x = $me->IBF;
  print "IBF2 @$x\n";
  @y = (2,28);
  $me->item("ibf",\@y);
  $x = $me->IBF;
  print "IBF3 @$x\n";
  my ($ovo,$rinto,$ov,$rint,$ind1,$ind2);
  $ovo = $me->item("overlap");
  $ov = $ovo->array(1);
  $ind1 = $ovo->make_indx(4,3);
  $ind2 = $ovo->make_indx(3,4);
  print "ind34 $ind1 $ind2\n";
  $rinto = $me->item("RAFFENETTI 2E INTEGRALS");
  if (defined($rinto)) {
    $rint = $rinto->array(1);
    $ind1 = $rinto->make_indx(4,3,2,1,2);
    $ind2 = $rinto->make_indx(1,2,3,4,2);
    print "ind1234 $ind1 $ind2\n";}
}
