#!/usr/bin/env perl
#
# Generate a matrix element file from data stored here, for testing.
#
use strict;
use MatEl;
use OpMat;
use GauComplex;

# scalars for header records
my (%headvals) = ('title' => "Generated matrix element file", 'natoms' => 3, 'nbasis' => 11,
		  'nbsuse' => 11, 'icharg' => 1, 'multip' => 2, 'ne' => 17, 'len12l' => 4,
		  'len4l' => 4, 'iopcl' => 6, 'icgu' => 221, 'nfc' => 0, 'nfv' => 0,
		  'itran' => 0, 'idum9' => 0);

# arrays for header records
my (@ian) = (8,9,1);
my (@iattyp) = (0,0,0);
my (@atmchg) = (8.0,9.0,1.0);
my (@c) = ( 0.0950213800722009e0, 1.30811041733782e0,0.0e0,
	    0.0950213800722015e0,-1.37530067954457e0,0.0e0,
	   -1.61536346122742e0,   1.91282277719863e0,0.0e0);
my (@ibf) = (1,1,1,1,1,2,2,2,2,2,3);
my (@ibftyp) = (1,1,1001,1002,1003,1,1,1001,1002,1003,1);
my (@atmwgt) = (15.99491464e0,18.99840325e0,1.007825037e0);

# map element names in MatEl structure to local arrays
my (%headarrs) = ('ian' => \@ian, 'iattyp' => \@iattyp, 'atmchg' => \@atmchg,
		  'c' => \@c, 'ibf' => \@ibf, 'ibftyp' => \@ibf, 'atmwgt' => \@atmwgt);

my ($me) = MatEl->new(1);

# generate the header data
my ($x,@args);
foreach $x (@MatEl::head_scalars) {push(@args,$headvals{$x});}
foreach $x (@MatEl::head_arrays) {push(@args,$headarrs{$x});}
$me->header(@args);

# scalars for /Gen/ from a GHF calculation on this system
$me->item("NSHELLAO",-100);
$me->item("NPRIMAO",-101);
$me->item("NSHELLDB",-102);
$me->item("NPRIMDB",-103);
$me->VIRIAL(2.02181581081603e0);
$me->TESCFENERGY(93.5527560008696e0);
$me->ENUCREP(33.6695831122157e0);
$me->ESCF(-172.042877315815e0);
$me->S2SCF(0.883193821743992e0);
$me->JOBSTATUS(1.0e0);
$me->SCFRMSDP(8.15053431869981e-09);
$me->ESCF1IT(-171.983768596442e0);
$me->ETOTAL(-172.042877315815e0);
my ($i,$x,$name) = (0);

# arrays from a GHF calculation on this system
&add1 ("ALPHA DENSITY MATRIX");
&add1 ("ALPHA FOCK MATRIX");
&add1 ("ALPHA MO COEFFICIENTS");
&add1 ("ALPHA ORBITAL ENERGIES");
&add1 ("ALPHA SCF DENSITY MATRIX");
&add1 ("ELECTRIC DIPOLE MOMENT");
&add1 ("ORTHOGONAL BASIS");

# write the file
$me->write($ARGV[5]);

# add a operator object to the MatEl structure using stored data
sub add1 {
  my ($name) = @_;
  my ($type,$dims,$arr) = &get_vals($name);
  my ($len);
  $len = @$arr;
  $me->addopmat($name,$type,$dims,$arr);
}

# return the parameters for a particular operator
sub get_vals {
  my ($name) = @_;
  my ($type) = gettype($name);
  my ($dims) = getdims($name);
  my ($at) = gettvals($name);
  my ($i,$x,@a) = (0);
# store the returned values, making sure that they are float or complex
  if ($type eq "c") {foreach $x (@$at) {$a[$i++] = GauComplex->make($x);}}
  else {foreach $x (@$at) {$a[$i++] = $x + 0.0e0;}}
  return ($type,$dims,\@a);
}

# return the type of a stored operator
sub gettype {
  my ($name) = @_;
  if ($name eq "ALPHA DENSITY MATRIX") {return("c");}
  elsif ($name eq "ALPHA MO COEFFICIENTS") {return("c");}
  elsif ($name eq "ALPHA FOCK MATRIX") {return("c");}
  elsif ($name eq "ALPHA ORBITAL ENERGIES") {return("d");}
  elsif ($name eq "ALPHA SCF DENSITY MATRIX") {return("c");}
  elsif ($name eq "ELECTRIC DIPOLE MOMENT") {return("d");}
  elsif ($name eq "ORTHOGONAL BASIS") {return("c");}
  else {die "bad $name in gettype.\n";}
}

# return the dimension of a stored operator
sub getdims {
  my ($name) = @_;
  my (@dims);
  if ($name eq "ALPHA DENSITY MATRIX") {@dims = (-22,22);}
  elsif ($name eq "ALPHA MO COEFFICIENTS") {@dims = (22,22);}
  elsif ($name eq "ALPHA FOCK MATRIX") {@dims = (-22,22);}
  elsif ($name eq "ALPHA ORBITAL ENERGIES") {@dims = (22);}
  elsif ($name eq "ALPHA SCF DENSITY MATRIX") {@dims = (-22,22);}
  elsif ($name eq "ELECTRIC DIPOLE MOMENT") {@dims = (3);}
  elsif ($name eq "ORTHOGONAL BASIS") {@dims = (22,22);}
  else {die "bad $name in getdims.\n";}
  return (\@dims);
}

# return the values for a stored operator.  complex
# value are returned as strings and must be converted.
sub gettvals {
  my ($name) = @_;
  my (@at);
  if ($name eq "ALPHA DENSITY MATRIX") {@at =("1.055321+0.000000i","-0.000000-0.001433i","1.055321+0.000000i","-0.233365+0.000000i","0.000000-0.007039i","1.007182+0.000000i","0.000000+0.007039i","-0.233365-0.000000i","-0.000000-0.032577i","1.007182-0.000000i","-0.032963-0.000000i","-0.000000+0.006703i","0.188444-0.000000i","0.000000-0.032241i","0.682020+0.000000i","-0.000000-0.006703i","-0.032963+0.000000i","0.000000+0.032241i","0.188444+0.000000i","-0.000000-0.027888i","0.682020-0.000000i","-0.017926-0.000000i","-0.000000+0.010250i","0.069982+0.000000i","0.000000-0.047905i","0.093058-0.000000i","-0.000000+0.016952i","0.545552-0.000000i","-0.000000-0.010250i","-0.017926-0.000000i","0.000000+0.047905i","0.069982+0.000000i","-0.000000-0.016952i","0.093058-0.000000i","-0.000000-0.183775i","0.545552-0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.515975-0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000-0.488679i","0.515975-0.000000i","-0.003610-0.000000i","0.000000-0.000927i","0.021658+0.000000i","-0.000000+0.004563i","-0.001295-0.000000i","0.000000-0.002072i","-0.006772-0.000000i","0.000000-0.012628i","0.000000+0.000000i","0.000000+0.000000i","1.060311-0.000000i","0.000000+0.000927i","-0.003610+0.000000i","-0.000000-0.004563i","0.021658-0.000000i","-0.000000+0.002072i","-0.001295-0.000000i","0.000000+0.012628i","-0.006772-0.000000i","0.000000+0.000000i","0.000000-0.000000i","-0.000000-0.000875i","1.060311+0.000000i","0.023562+0.000000i","0.000000+0.004527i","-0.126638+0.000000i","0.000000-0.022206i","0.009734-0.000000i","-0.000000+0.010219i","0.071373+0.000000i","-0.000000+0.063878i","0.000000-0.000000i","0.000000-0.000000i","-0.255231-0.000000i","0.000000-0.004439i","1.084940+0.000000i","-0.000000-0.004527i","0.023562-0.000000i","0.000000+0.022206i","-0.126638+0.000000i","-0.000000-0.010219i","0.009734+0.000000i","-0.000000-0.063878i","0.071373+0.000000i","-0.000000-0.000000i","-0.000000-0.000000i","0.000000+0.004439i","-0.255231-0.000000i","-0.000000-0.022483i","1.084940+0.000000i","0.003866-0.000000i","0.000000-0.001627i","-0.027083+0.000000i","-0.000000+0.008463i","-0.038056+0.000000i","-0.000000-0.003592i","-0.009264-0.000000i","0.000000-0.012280i","-0.000000-0.000000i","0.000000+0.000000i","0.000090+0.000000i","-0.000000+0.000804i","-0.000980-0.000000i","0.000000-0.004217i","1.002986+0.000000i","-0.000000+0.001627i","0.003866+0.000000i","-0.000000-0.008463i","-0.027083+0.000000i","0.000000+0.003592i","-0.038056+0.000000i","0.000000+0.012280i","-0.009264-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","-0.000000-0.000804i","0.000090+0.000000i","-0.000000+0.004217i","-0.000980-0.000000i","-0.000000-0.000196i","1.002986+0.000000i","-0.018858-0.000000i","0.000000-0.001538i","0.063388+0.000000i","-0.000000+0.012841i","-0.034402-0.000000i","-0.000000+0.021003i","-0.351253-0.000000i","-0.000000+0.045420i","-0.000000-0.000000i","-0.000000+0.000000i","0.021802+0.000000i","0.000000-0.005157i","-0.106431-0.000000i","-0.000000+0.023185i","-0.002383+0.000000i","-0.000000-0.016828i","0.308355+0.000000i","-0.000000+0.001538i","-0.018858+0.000000i","-0.000000-0.012841i","0.063388+0.000000i","-0.000000-0.021003i","-0.034402-0.000000i","-0.000000-0.045420i","-0.351253-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","0.000000+0.005157i","0.021802-0.000000i","-0.000000-0.023185i","-0.106431-0.000000i","0.000000+0.016828i","-0.002383+0.000000i","0.000000+0.171580i","0.308355+0.000000i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.046359-0.000000i","0.000000-0.114737i","-0.000000+0.000000i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000-0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.977715-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","0.000000+0.114737i","0.046359-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000-0.026939i","0.977715+0.000000i","-0.014399-0.000000i","0.000000+0.005338i","-0.002506+0.000000i","0.000000-0.017713i","-0.291788+0.000000i","-0.000000+0.015537i","0.074997-0.000000i","-0.000000+0.014726i","-0.000000-0.000000i","-0.000000+0.000000i","-0.001019+0.000000i","0.000000-0.002203i","0.005290-0.000000i","-0.000000+0.010822i","0.052069+0.000000i","0.000000-0.004360i","-0.055074-0.000000i","-0.000000+0.006025i","0.000000-0.000000i","-0.000000+0.000000i","0.158748-0.000000i","-0.000000-0.005338i","-0.014399+0.000000i","0.000000+0.017713i","-0.002506+0.000000i","0.000000-0.015537i","-0.291788+0.000000i","-0.000000-0.014726i","0.074997-0.000000i","0.000000+0.000000i","0.000000-0.000000i","0.000000+0.002203i","-0.001019-0.000000i","-0.000000-0.010822i","0.005290-0.000000i","0.000000+0.004360i","0.052069+0.000000i","-0.000000-0.006025i","-0.055074-0.000000i","-0.000000-0.000000i","0.000000+0.000000i","0.000000+0.023606i","0.158748+0.000000i");}
  elsif ($name eq "ALPHA MO COEFFICIENTS") {@at = ("0.000203-0.000000i","-0.000000-0.000203i","-0.001404+0.000000i","0.000000+0.001404i","-0.000081-0.000000i","0.000000+0.000081i","0.001690-0.000000i","-0.000000-0.001690i","0.000000+0.000000i","0.000000-0.000000i","0.703599+0.000000i","-0.000000-0.703599i","0.014367-0.000000i","-0.000000-0.014367i","-0.000029+0.000000i","0.000000+0.000029i","0.001049-0.000000i","-0.000000-0.001049i","0.000000-0.000000i","-0.000000+0.000000i","-0.000217-0.000000i","0.000000+0.000217i","0.000193+0.000000i","0.000000+0.000193i","-0.001415-0.000000i","-0.000000-0.001415i","-0.000070-0.000000i","0.000000-0.000070i","0.001704-0.000000i","0.000000+0.001704i","-0.000000-0.000000i","0.000000+0.000000i","0.703696+0.000000i","0.000000+0.703696i","0.013988+0.000000i","0.000000+0.013988i","-0.000032-0.000000i","-0.000000-0.000032i","0.001066+0.000000i","0.000000+0.001066i","-0.000000-0.000000i","0.000000-0.000000i","-0.000205-0.000000i","-0.000000-0.000205i","0.703056+0.000000i","-0.000000+0.703056i","0.017612-0.000000i","-0.000000+0.017612i","-0.002514-0.000000i","0.000000-0.002514i","-0.000422-0.000000i","-0.000000-0.000422i","0.000000+0.000000i","0.000000-0.000000i","0.000324+0.000000i","-0.000000+0.000324i","-0.001680-0.000000i","0.000000-0.001680i","0.000025+0.000000i","0.000000+0.000025i","-0.001573+0.000000i","0.000000-0.001573i","0.000000+0.000000i","-0.000000+0.000000i","-0.004612+0.000000i","0.000000-0.004612i","0.703569+0.000000i","0.000000-0.703569i","0.015459-0.000000i","0.000000-0.015459i","-0.002261+0.000000i","-0.000000+0.002261i","-0.000369-0.000000i","-0.000000+0.000369i","-0.000000+0.000000i","-0.000000+0.000000i","0.000268+0.000000i","-0.000000-0.000268i","-0.001445-0.000000i","-0.000000+0.001445i","0.000019+0.000000i","-0.000000-0.000019i","-0.001362+0.000000i","-0.000000+0.001362i","-0.000000+0.000000i","0.000000+0.000000i","-0.004071-0.000000i","-0.000000+0.004071i","-0.123760+0.000000i","0.000000-0.123760i","0.458361+0.000000i","-0.000000+0.458361i","-0.062807+0.000000i","0.000000-0.062807i","-0.079666+0.000000i","0.000000-0.079666i","-0.000000-0.000000i","0.000000-0.000000i","-0.119812+0.000000i","0.000000-0.119812i","0.443993-0.000000i","-0.000000+0.443993i","-0.008385+0.000000i","0.000000-0.008385i","0.060666-0.000000i","-0.000000+0.060666i","-0.000000-0.000000i","-0.000000-0.000000i","0.033274-0.000000i","-0.000000+0.033274i","-0.066263+0.000000i","-0.000000+0.066263i","0.233685-0.000000i","0.000000-0.233685i","-0.026910+0.000000i","-0.000000+0.026910i","-0.068749+0.000000i","-0.000000+0.068749i","0.000000-0.000000i","0.000000+0.000000i","-0.163378+0.000000i","-0.000000+0.163378i","0.622558+0.000000i","0.000000-0.622558i","-0.007011+0.000000i","-0.000000+0.007011i","0.062814-0.000000i","0.000000-0.062814i","-0.000000+0.000000i","-0.000000-0.000000i","0.024945-0.000000i","0.000000-0.024945i","0.126401+0.000000i","-0.000000+0.126401i","-0.490715-0.000000i","0.000000-0.490715i","0.092914+0.000000i","-0.000000+0.092914i","-0.076794+0.000000i","0.000000-0.076794i","0.000000-0.000000i","0.000000+0.000000i","-0.134710+0.000000i","0.000000-0.134710i","0.541454+0.000000i","-0.000000+0.541454i","-0.002426+0.000000i","-0.000000-0.002426i","-0.009291+0.000000i","0.000000-0.009291i","0.000000+0.000000i","0.000000+0.000000i","-0.071668+0.000000i","0.000000-0.071668i","-0.151547-0.000000i","-0.000000+0.151547i","0.570623+0.000000i","0.000000-0.570623i","-0.110659+0.000000i","-0.000000+0.110659i","0.045243-0.000000i","0.000000-0.045243i","0.000000+0.000000i","0.000000-0.000000i","0.082144-0.000000i","0.000000-0.082144i","-0.342186+0.000000i","-0.000000+0.342186i","-0.006047+0.000000i","-0.000000+0.006047i","0.087231+0.000000i","0.000000-0.087231i","0.000000+0.000000i","-0.000000+0.000000i","0.130679+0.000000i","0.000000-0.130679i","-0.044835-0.000000i","0.000000-0.044835i","0.226467+0.000000i","-0.000000+0.226467i","0.476308+0.000000i","-0.000000+0.476308i","-0.212529-0.000000i","0.000000-0.212529i","-0.000000+0.000000i","-0.000000-0.000000i","0.030783+0.000000i","-0.000000+0.030783i","-0.140036-0.000000i","0.000000-0.140036i","0.188061-0.000000i","0.000000+0.188061i","0.118243+0.000000i","-0.000000+0.118243i","-0.000000+0.000000i","-0.000000-0.000000i","-0.226811-0.000000i","0.000000-0.226811i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.646329+0.000000i","-0.000000+0.646329i","0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.246171+0.000000i","-0.000000+0.246171i","-0.000000-0.000000i","0.000000-0.000000i","-0.037364-0.000000i","-0.000000+0.037364i","0.189673+0.000000i","0.000000-0.189673i","0.398448+0.000000i","0.000000-0.398448i","-0.190660+0.000000i","-0.000000+0.190660i","-0.000000+0.000000i","0.000000-0.000000i","0.026305-0.000000i","0.000000-0.026305i","-0.119731+0.000000i","-0.000000+0.119731i","0.240802-0.000000i","0.000000-0.240802i","0.267417-0.000000i","0.000000-0.267417i","-0.000000+0.000000i","-0.000000-0.000000i","-0.228385+0.000000i","-0.000000+0.228385i","-0.019745+0.000000i","0.000000-0.019745i","0.082966-0.000000i","-0.000000+0.082966i","0.234655+0.000000i","-0.000000+0.234655i","0.538885+0.000000i","-0.000000+0.538885i","-0.000000+0.000000i","-0.000000-0.000000i","-0.044046-0.000000i","0.000000-0.044046i","0.207588+0.000000i","-0.000000+0.207588i","0.209153-0.000000i","-0.000000+0.209153i","-0.223766+0.000000i","0.000000-0.223766i","-0.000000+0.000000i","0.000000-0.000000i","0.012242-0.000000i","-0.000000+0.012242i","-0.033580+0.000000i","-0.000000+0.033580i","0.146193-0.000000i","0.000000-0.146193i","0.177549-0.000000i","0.000000-0.177549i","0.332300-0.000000i","0.000000-0.332300i","-0.000000+0.000000i","-0.000000+0.000000i","-0.022840+0.000000i","-0.000000+0.022840i","0.107669-0.000000i","0.000000-0.107669i","0.405023+0.000000i","0.000000-0.405023i","-0.358516+0.000000i","-0.000000+0.358516i","-0.000000-0.000000i","0.000000+0.000000i","0.019017-0.000000i","0.000000-0.019017i","-0.000000+0.000000i","0.000000+0.000000i","0.000000-0.000000i","-0.000000-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.116824-0.000000i","0.000000-0.116824i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000-0.000000i","0.000000+0.000000i","0.689484+0.000000i","0.000000-0.689484i","-0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","-0.290835+0.000000i","0.000000-0.290835i","-0.000000+0.000000i","0.000000-0.000000i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.664625+0.000000i","-0.000000+0.664625i","-0.000000-0.000000i","0.000000-0.000000i","0.019897-0.000000i","-0.000000+0.019897i","-0.102473+0.000000i","0.000000-0.102473i","-0.245842-0.000000i","0.000000-0.245842i","-0.129810-0.000000i","0.000000-0.129810i","-0.000000+0.000000i","-0.000000-0.000000i","0.003941+0.000000i","-0.000000+0.003941i","-0.022531-0.000000i","0.000000-0.022531i","0.649925+0.000000i","-0.000000+0.649925i","0.023764-0.000000i","-0.000000+0.023764i","0.000000-0.000000i","0.000000+0.000000i","0.098556+0.000000i","-0.000000+0.098556i","0.045320+0.000000i","0.000000-0.045320i","-0.222472-0.000000i","-0.000000+0.222472i","-0.351862-0.000000i","-0.000000+0.351862i","-0.165342-0.000000i","-0.000000+0.165342i","0.000000+0.000000i","-0.000000+0.000000i","0.003655+0.000000i","0.000000-0.003655i","-0.020554-0.000000i","-0.000000+0.020554i","0.528469+0.000000i","0.000000-0.528469i","0.168417+0.000000i","0.000000-0.168417i","-0.000000-0.000000i","0.000000-0.000000i","0.144707+0.000000i","0.000000-0.144707i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000+0.000000i","0.000000-0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.699056+0.000000i","0.000000-0.699056i","0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.164132+0.000000i","-0.000000+0.164132i","0.000000-0.000000i","-0.000000+0.000000i","0.056573-0.000000i","0.000000-0.056573i","-0.294263+0.000000i","-0.000000+0.294263i","0.116551+0.000000i","0.000000-0.116551i","0.556377+0.000000i","0.000000-0.556377i","0.000000-0.000000i","-0.000000-0.000000i","-0.036336-0.000000i","-0.000000+0.036336i","0.188543+0.000000i","0.000000-0.188543i","-0.016530-0.000000i","-0.000000+0.016530i","0.548601+0.000000i","0.000000-0.548601i","-0.000000-0.000000i","0.000000+0.000000i","0.159796-0.000000i","0.000000-0.159796i","0.049622-0.000000i","-0.000000+0.049622i","-0.271497+0.000000i","0.000000-0.271497i","0.084423-0.000000i","-0.000000+0.084423i","0.370448+0.000000i","-0.000000+0.370448i","-0.000000-0.000000i","-0.000000+0.000000i","-0.021255-0.000000i","0.000000-0.021255i","0.115140+0.000000i","-0.000000+0.115140i","0.010200-0.000000i","0.000000+0.010200i","0.683313+0.000000i","-0.000000+0.683313i","-0.000000+0.000000i","0.000000+0.000000i","0.161017-0.000000i","-0.000000+0.161017i","0.054326-0.000000i","-0.000000+0.054326i","-0.399121+0.000000i","0.000000-0.399121i","0.500790-0.000000i","-0.000000+0.500790i","-0.253259-0.000000i","0.000000-0.253259i","-0.000000+0.000000i","0.000000+0.000000i","0.008597-0.000000i","-0.000000+0.008597i","-0.052439-0.000000i","0.000000-0.052439i","-0.025518+0.000000i","0.000000-0.025518i","-0.103011-0.000000i","0.000000-0.103011i","0.000000-0.000000i","-0.000000-0.000000i","0.845753+0.000000i","-0.000000+0.845753i","0.060384+0.000000i","0.000000-0.060384i","-0.423078-0.000000i","-0.000000+0.423078i","0.521749+0.000000i","0.000000-0.521749i","-0.274940-0.000000i","-0.000000+0.274940i","0.000000-0.000000i","-0.000000-0.000000i","0.008958+0.000000i","0.000000-0.008958i","-0.054233-0.000000i","-0.000000+0.054233i","-0.026034-0.000000i","-0.000000+0.026034i","-0.070603-0.000000i","-0.000000+0.070603i","0.000000+0.000000i","-0.000000-0.000000i","0.831915+0.000000i","0.000000-0.831915i");}
  elsif ($name eq "ALPHA FOCK MATRIX") {@at = ("-21.050494+0.000000i","0.000000+0.018483i","-21.050494+0.000000i","-5.341642-0.000000i","0.000000-0.025638i","-3.037664-0.000000i","0.000000+0.025638i","-5.341642+0.000000i","0.000000+0.124574i","-3.037664-0.000000i","0.021413+0.000000i","0.000000+0.000280i","0.123717+0.000000i","-0.000000+0.012524i","-1.022265+0.000000i","-0.000000-0.000280i","0.021413-0.000000i","-0.000000-0.012524i","0.123717-0.000000i","0.000000+0.046132i","-1.022265+0.000000i","0.009723+0.000000i","-0.000000-0.000206i","0.041873-0.000000i","-0.000000+0.021323i","0.010907+0.000000i","0.000000-0.009728i","-0.919436+0.000000i","0.000000+0.000206i","0.009723+0.000000i","-0.000000-0.021323i","0.041873-0.000000i","0.000000+0.009728i","0.010907+0.000000i","0.000000+0.155104i","-0.919436+0.000000i","0.000000+0.000000i","0.000000-0.000000i","-0.000000-0.000000i","0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.703041-0.000000i","0.000000+0.000000i","0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","0.000000-0.000000i","0.000000+0.000000i","0.000000-0.000000i","0.000000+0.000000i","0.000000+0.433322i","-0.703041-0.000000i","-0.000059-0.000000i","0.000000+0.000021i","-0.259907-0.000000i","0.000000+0.000688i","0.000127+0.000000i","-0.000000-0.000063i","0.476569+0.000000i","-0.000000-0.001008i","-0.000000+0.000000i","-0.000000+0.000000i","-26.578933-0.000000i","-0.000000-0.000021i","-0.000059-0.000000i","0.000000-0.000688i","-0.259907+0.000000i","0.000000+0.000063i","0.000127+0.000000i","0.000000+0.001008i","0.476569+0.000000i","0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.003055i","-26.578933+0.000000i","-0.147613-0.000000i","0.000000-0.000741i","-0.500622-0.000000i","0.000000-0.000313i","0.006688-0.000000i","0.000000-0.000623i","0.632376+0.000000i","0.000000-0.005740i","-0.000000+0.000000i","-0.000000+0.000000i","-6.773264+0.000000i","-0.000000+0.006634i","-3.544322-0.000000i","-0.000000+0.000741i","-0.147613-0.000000i","-0.000000+0.000313i","-0.500622-0.000000i","0.000000+0.000623i","0.006688+0.000000i","0.000000+0.005740i","0.632376+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.000000-0.006634i","-6.773264+0.000000i","-0.000000-0.027024i","-3.544322-0.000000i","0.000124+0.000000i","0.000000-0.000038i","0.004813-0.000000i","0.000000-0.000633i","-0.146058-0.000000i","0.000000-0.000006i","-0.004317+0.000000i","-0.000000+0.001984i","0.000000+0.000000i","0.000000-0.000000i","0.000845-0.000000i","0.000000-0.000007i","0.011016+0.000000i","-0.000000+0.001554i","-0.987998-0.000000i","0.000000+0.000038i","0.000124-0.000000i","0.000000+0.000633i","0.004813-0.000000i","-0.000000+0.000006i","-0.146058-0.000000i","-0.000000-0.001984i","-0.004317+0.000000i","0.000000+0.000000i","-0.000000-0.000000i","-0.000000+0.000007i","0.000845-0.000000i","0.000000-0.001554i","0.011016-0.000000i","-0.000000-0.006344i","-0.987998-0.000000i","-0.293415+0.000000i","0.000000-0.001553i","-0.527613-0.000000i","0.000000-0.001059i","0.015145+0.000000i","-0.000000-0.003050i","0.552351+0.000000i","0.000000-0.013645i","0.000000+0.000000i","0.000000-0.000000i","-0.015980+0.000000i","-0.000000-0.000193i","-0.085292+0.000000i","0.000000-0.014132i","0.005636-0.000000i","-0.000000+0.012723i","-0.669238+0.000000i","0.000000+0.001553i","-0.293415-0.000000i","-0.000000+0.001059i","-0.527613-0.000000i","0.000000+0.003050i","0.015145+0.000000i","0.000000+0.013645i","0.552351+0.000000i","0.000000-0.000000i","-0.000000-0.000000i","-0.000000+0.000193i","-0.015980+0.000000i","0.000000+0.014132i","-0.085292+0.000000i","-0.000000-0.012723i","0.005636-0.000000i","-0.000000-0.170547i","-0.669238+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.166949+0.000000i","-0.000000+0.021399i","0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.000000-0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.976026-0.000000i","-0.000000-0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000-0.000000i","-0.000000-0.021399i","-0.166949+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","0.000000+0.011101i","-0.976026+0.000000i","-1.227322-0.000000i","0.000000-0.006797i","-1.271762-0.000000i","0.000000-0.031997i","0.716680-0.000000i","0.000000+0.004274i","-0.204296+0.000000i","0.000000-0.005737i","0.000000+0.000000i","0.000000-0.000000i","-0.131503+0.000000i","-0.000000+0.000273i","-0.204129+0.000000i","-0.000000-0.000175i","0.058413-0.000000i","0.000000+0.000236i","-0.153530+0.000000i","-0.000000+0.000619i","0.000000+0.000000i","0.000000-0.000000i","-0.995050-0.000000i","0.000000+0.006797i","-1.227322-0.000000i","0.000000+0.031997i","-1.271762-0.000000i","-0.000000-0.004274i","0.716680-0.000000i","0.000000+0.005737i","-0.204296+0.000000i","0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.000273i","-0.131503+0.000000i","0.000000+0.000175i","-0.204129+0.000000i","-0.000000-0.000236i","0.058413-0.000000i","0.000000-0.000619i","-0.153530+0.000000i","0.000000-0.000000i","-0.000000-0.000000i","-0.000000-0.005382i","-0.995050-0.000000i");}
  elsif ($name eq "ALPHA ORBITAL ENERGIES") {@at = (-26.5910760227626e0,-26.5844970801773e0,-21.077744444113e0,-21.0388491999089e0,-2.05316587899934e0,-2.01791437112064e0,-1.77436402664192e0,-1.645147326701e0,-1.16862237665815e0,-1.16168467612092e0,-1.13438018542531e0,-1.07369620731101e0,-1.02108532663628e0,-0.985473187691492e0,-0.951782614626619e0,-0.946122337973426e0,-0.934099185914718e0,-0.229158871046169e0,-0.110236889993061e0,-0.0828253167721301e0,0.12559637606487e0,0.160485671510934e0);}
  elsif ($name eq "ALPHA SCF DENSITY MATRIX") {@at = ("1.055321+0.000000i","-0.000000-0.001433i","1.055321+0.000000i","-0.233365+0.000000i","0.000000-0.007039i","1.007182+0.000000i","0.000000+0.007039i","-0.233365-0.000000i","-0.000000-0.032577i","1.007182-0.000000i","-0.032963-0.000000i","-0.000000+0.006703i","0.188444-0.000000i","0.000000-0.032241i","0.682020+0.000000i","-0.000000-0.006703i","-0.032963+0.000000i","0.000000+0.032241i","0.188444+0.000000i","-0.000000-0.027888i","0.682020-0.000000i","-0.017926-0.000000i","-0.000000+0.010250i","0.069982+0.000000i","0.000000-0.047905i","0.093058-0.000000i","-0.000000+0.016952i","0.545552-0.000000i","-0.000000-0.010250i","-0.017926-0.000000i","0.000000+0.047905i","0.069982+0.000000i","-0.000000-0.016952i","0.093058-0.000000i","-0.000000-0.183775i","0.545552-0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.515975-0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000-0.488679i","0.515975-0.000000i","-0.003610-0.000000i","0.000000-0.000927i","0.021658+0.000000i","-0.000000+0.004563i","-0.001295-0.000000i","0.000000-0.002072i","-0.006772-0.000000i","0.000000-0.012628i","0.000000+0.000000i","0.000000+0.000000i","1.060311-0.000000i","0.000000+0.000927i","-0.003610+0.000000i","-0.000000-0.004563i","0.021658-0.000000i","-0.000000+0.002072i","-0.001295-0.000000i","0.000000+0.012628i","-0.006772-0.000000i","0.000000+0.000000i","0.000000-0.000000i","-0.000000-0.000875i","1.060311+0.000000i","0.023562+0.000000i","0.000000+0.004527i","-0.126638+0.000000i","0.000000-0.022206i","0.009734-0.000000i","-0.000000+0.010219i","0.071373+0.000000i","-0.000000+0.063878i","0.000000-0.000000i","0.000000-0.000000i","-0.255231-0.000000i","0.000000-0.004439i","1.084940+0.000000i","-0.000000-0.004527i","0.023562-0.000000i","0.000000+0.022206i","-0.126638+0.000000i","-0.000000-0.010219i","0.009734+0.000000i","-0.000000-0.063878i","0.071373+0.000000i","-0.000000-0.000000i","-0.000000-0.000000i","0.000000+0.004439i","-0.255231-0.000000i","-0.000000-0.022483i","1.084940+0.000000i","0.003866-0.000000i","0.000000-0.001627i","-0.027083+0.000000i","-0.000000+0.008463i","-0.038056+0.000000i","-0.000000-0.003592i","-0.009264-0.000000i","0.000000-0.012280i","-0.000000-0.000000i","0.000000+0.000000i","0.000090+0.000000i","-0.000000+0.000804i","-0.000980-0.000000i","0.000000-0.004217i","1.002986+0.000000i","-0.000000+0.001627i","0.003866+0.000000i","-0.000000-0.008463i","-0.027083+0.000000i","0.000000+0.003592i","-0.038056+0.000000i","0.000000+0.012280i","-0.009264-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","-0.000000-0.000804i","0.000090+0.000000i","-0.000000+0.004217i","-0.000980-0.000000i","-0.000000-0.000196i","1.002986+0.000000i","-0.018858-0.000000i","0.000000-0.001538i","0.063388+0.000000i","-0.000000+0.012841i","-0.034402-0.000000i","-0.000000+0.021003i","-0.351253-0.000000i","-0.000000+0.045420i","-0.000000-0.000000i","-0.000000+0.000000i","0.021802+0.000000i","0.000000-0.005157i","-0.106431-0.000000i","-0.000000+0.023185i","-0.002383+0.000000i","-0.000000-0.016828i","0.308355+0.000000i","-0.000000+0.001538i","-0.018858+0.000000i","-0.000000-0.012841i","0.063388+0.000000i","-0.000000-0.021003i","-0.034402-0.000000i","-0.000000-0.045420i","-0.351253-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","0.000000+0.005157i","0.021802-0.000000i","-0.000000-0.023185i","-0.106431-0.000000i","0.000000+0.016828i","-0.002383+0.000000i","0.000000+0.171580i","0.308355+0.000000i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.046359-0.000000i","0.000000-0.114737i","-0.000000+0.000000i","0.000000-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000-0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.000000+0.000000i","0.977715-0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","-0.000000+0.000000i","0.000000-0.000000i","0.000000+0.114737i","0.046359-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","0.000000-0.000000i","0.000000+0.000000i","-0.000000+0.000000i","-0.000000+0.000000i","-0.000000-0.000000i","-0.000000-0.026939i","0.977715+0.000000i","-0.014399-0.000000i","0.000000+0.005338i","-0.002506+0.000000i","0.000000-0.017713i","-0.291788+0.000000i","-0.000000+0.015537i","0.074997-0.000000i","-0.000000+0.014726i","-0.000000-0.000000i","-0.000000+0.000000i","-0.001019+0.000000i","0.000000-0.002203i","0.005290-0.000000i","-0.000000+0.010822i","0.052069+0.000000i","0.000000-0.004360i","-0.055074-0.000000i","-0.000000+0.006025i","0.000000-0.000000i","-0.000000+0.000000i","0.158748-0.000000i","-0.000000-0.005338i","-0.014399+0.000000i","0.000000+0.017713i","-0.002506+0.000000i","0.000000-0.015537i","-0.291788+0.000000i","-0.000000-0.014726i","0.074997-0.000000i","0.000000+0.000000i","0.000000-0.000000i","0.000000+0.002203i","-0.001019-0.000000i","-0.000000-0.010822i","0.005290-0.000000i","0.000000+0.004360i","0.052069+0.000000i","-0.000000-0.006025i","-0.055074-0.000000i","-0.000000-0.000000i","0.000000+0.000000i","0.000000+0.023606i","0.158748+0.000000i");}
  elsif ($name eq "ELECTRIC DIPOLE MOMENT") {@at = (-0.84746905493695e0,0.800939095039385e0,0.0e0);}
  elsif ($name eq "ORTHOGONAL BASIS") {@at = ("0.190592+0.000000i","0.000000+0.000000i","0.451905+0.000000i","0.000000+0.000000i","-0.251373+0.000000i","0.000000+0.000000i","-0.006674+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.062344+0.000000i","0.000000+0.000000i","0.159709+0.000000i","0.000000+0.000000i","-0.044126+0.000000i","0.000000+0.000000i","0.156576+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.474407+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.190592+0.000000i","0.000000+0.000000i","0.451905+0.000000i","0.000000+0.000000i","-0.251373+0.000000i","0.000000+0.000000i","-0.006674+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.062344+0.000000i","0.000000+0.000000i","0.159709+0.000000i","0.000000+0.000000i","-0.044126+0.000000i","0.000000+0.000000i","0.156576+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.474407+0.000000i","-0.017151+0.000000i","0.000000+0.000000i","-0.049560+0.000000i","0.000000+0.000000i","-0.229170+0.000000i","0.000000+0.000000i","0.496957+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.309427+0.000000i","0.000000+0.000000i","-0.449796+0.000000i","0.000000+0.000000i","-0.059796+0.000000i","0.000000+0.000000i","-0.279477+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.218427+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.017151+0.000000i","0.000000+0.000000i","-0.049560+0.000000i","0.000000+0.000000i","-0.229170+0.000000i","0.000000+0.000000i","0.496957+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.309427+0.000000i","0.000000+0.000000i","-0.449796+0.000000i","0.000000+0.000000i","-0.059796+0.000000i","0.000000+0.000000i","-0.279477+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.218427+0.000000i","-0.303352+0.000000i","0.000000+0.000000i","-0.224356+0.000000i","0.000000+0.000000i","-0.311563+0.000000i","0.000000+0.000000i","0.154745+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.497311+0.000000i","0.000000+0.000000i","0.362572+0.000000i","0.000000+0.000000i","-0.148489+0.000000i","0.000000+0.000000i","-0.425800+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.111984+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.303352+0.000000i","0.000000+0.000000i","-0.224356+0.000000i","0.000000+0.000000i","-0.311563+0.000000i","0.000000+0.000000i","0.154745+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.497311+0.000000i","0.000000+0.000000i","0.362572+0.000000i","0.000000+0.000000i","-0.148489+0.000000i","0.000000+0.000000i","-0.425800+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.111984+0.000000i","0.438152+0.000000i","0.000000+0.000000i","0.230052+0.000000i","0.000000+0.000000i","0.454236+0.000000i","0.000000+0.000000i","0.376112+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.241477+0.000000i","0.000000+0.000000i","0.126789+0.000000i","0.000000+0.000000i","0.310875+0.000000i","0.000000+0.000000i","-0.366831+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.073621+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.438152+0.000000i","0.000000+0.000000i","0.230052+0.000000i","0.000000+0.000000i","0.454236+0.000000i","0.000000+0.000000i","0.376112+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.241477+0.000000i","0.000000+0.000000i","0.126789+0.000000i","0.000000+0.000000i","0.310875+0.000000i","0.000000+0.000000i","-0.366831+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.073621+0.000000i","-0.393125+0.000000i","0.000000+0.000000i","-0.016973+0.000000i","0.000000+0.000000i","-0.125301+0.000000i","0.000000+0.000000i","0.012899+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.011522+0.000000i","0.000000+0.000000i","-0.000818+0.000000i","0.000000+0.000000i","0.899187+0.000000i","0.000000+0.000000i","0.099839+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.160368+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.393125+0.000000i","0.000000+0.000000i","-0.016973+0.000000i","0.000000+0.000000i","-0.125301+0.000000i","0.000000+0.000000i","0.012899+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.011522+0.000000i","0.000000+0.000000i","-0.000818+0.000000i","0.000000+0.000000i","0.899187+0.000000i","0.000000+0.000000i","0.099839+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.160368+0.000000i","0.595718+0.000000i","0.000000+0.000000i","-0.259275+0.000000i","0.000000+0.000000i","-0.497673+0.000000i","0.000000+0.000000i","-0.422853+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.233719+0.000000i","0.000000+0.000000i","0.101085+0.000000i","0.000000+0.000000i","0.252460+0.000000i","0.000000+0.000000i","-0.382639+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.115546+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.595718+0.000000i","0.000000+0.000000i","-0.259275+0.000000i","0.000000+0.000000i","-0.497673+0.000000i","0.000000+0.000000i","-0.422853+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.233719+0.000000i","0.000000+0.000000i","0.101085+0.000000i","0.000000+0.000000i","0.252460+0.000000i","0.000000+0.000000i","-0.382639+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.115546+0.000000i","0.354309+0.000000i","0.000000+0.000000i","-0.251505+0.000000i","0.000000+0.000000i","-0.280556+0.000000i","0.000000+0.000000i","0.162857+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.675271+0.000000i","0.000000+0.000000i","-0.482920+0.000000i","0.000000+0.000000i","0.087072+0.000000i","0.000000+0.000000i","0.460344+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.119133+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.354309+0.000000i","0.000000+0.000000i","-0.251505+0.000000i","0.000000+0.000000i","-0.280556+0.000000i","0.000000+0.000000i","0.162857+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.675271+0.000000i","0.000000+0.000000i","-0.482920+0.000000i","0.000000+0.000000i","0.087072+0.000000i","0.000000+0.000000i","0.460344+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.119133+0.000000i","0.200154+0.000000i","0.000000+0.000000i","-0.342410+0.000000i","0.000000+0.000000i","-0.204024+0.000000i","0.000000+0.000000i","0.695794+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.381745+0.000000i","0.000000+0.000000i","0.718842+0.000000i","0.000000+0.000000i","0.017754+0.000000i","0.000000+0.000000i","0.541762+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.221546+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.200154+0.000000i","0.000000+0.000000i","-0.342410+0.000000i","0.000000+0.000000i","-0.204024+0.000000i","0.000000+0.000000i","0.695794+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.381745+0.000000i","0.000000+0.000000i","0.718842+0.000000i","0.000000+0.000000i","0.017754+0.000000i","0.000000+0.000000i","0.541762+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.221546+0.000000i","0.268214+0.000000i","0.000000+0.000000i","-0.973566+0.000000i","0.000000+0.000000i","0.650322+0.000000i","0.000000+0.000000i","-0.173297+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.022331+0.000000i","0.000000+0.000000i","0.062571+0.000000i","0.000000+0.000000i","-0.017516+0.000000i","0.000000+0.000000i","0.107397+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","1.106577+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.268214+0.000000i","0.000000+0.000000i","-0.973566+0.000000i","0.000000+0.000000i","0.650322+0.000000i","0.000000+0.000000i","-0.173297+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.022331+0.000000i","0.000000+0.000000i","0.062571+0.000000i","0.000000+0.000000i","-0.017516+0.000000i","0.000000+0.000000i","0.107397+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","1.106577+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.684206+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.684206+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.684206+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.684206+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.732473+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.732473+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.732473+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","0.000000+0.000000i","-0.732473+0.000000i","0.000000+0.000000i","0.000000+0.000000i");}
  else {die "bad $name in gettvals.\n";}
  return (\@at);
}
